/*
 *  Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Sun Microsystems, Inc. for Project JXTA."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact Project JXTA at http://www.jxta.org.
 *
 *  5. Products derived from this software may not be called "JXTA",
 *  nor may "JXTA" appear in their name, without prior written
 *  permission of Sun.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *  ====================================================================
 *
 *  This software consists of voluntary contributions made by many
 *  individuals on behalf of Project JXTA.  For more
 *  information on Project JXTA, please see
 *  <http://www.jxta.org/>.
 *
 *  This license is based on the BSD license adopted by the Apache Foundation.
 *
 *  $Id: Configurator.java,v 1.69 2002/01/29 01:41:49 bondolo Exp $
 */
package net.jxta.impl.peergroup;

import java.net.*;
import java.util.*;
import java.io.*;


import java.awt.TextField;
import java.awt.event.*;

import net.jxta.protocol.PeerAdvertisement;
import net.jxta.protocol.TransportAdvertisement;
import net.jxta.impl.protocol.HTTPAdv;
import net.jxta.impl.protocol.TCPAdv;
import net.jxta.impl.endpoint.tls.TlsConfig;
import net.jxta.impl.endpoint.tls.JTlsUtil;

import net.jxta.peer.*;
import net.jxta.peergroup.*;
import net.jxta.resolver.*;
import net.jxta.id.*;
import net.jxta.exception.*;
import net.jxta.credential.*;
import net.jxta.document.*;

import org.apache.log4j.Category;
import org.apache.log4j.Priority;

/**
 *Description of the Class
 */
public class Configurator {
    
    static String configName = "PlatformConfig";
    private final static Category LOG =
    Category.getInstance(Configurator.class.getName());
    
    PeerAdvertisement advertisement = null;
    
    
    /**
     * Constructor for the Configurator object
     *
     * @since 1.0
     */
    public Configurator() {
        
        // start with error as the default
        //LOG.setPriority(Priority.toPriority("debug"));
        
        load();
        
        // Make sure it does not need a reconf
        if (fixAdvertisement() == false) {
            adjustLog4JPriority();
            
            // if there's a password set, or a pass
            // pop a dialog to get the password
            checkPassword();
            
            // Set Tls password, and check for
            // changed password.
            configureTls();
            
            routerRdvMatch();
            return;
        }
        
        try {
            ConfigDialog ignore = new ConfigDialog(advertisement);
            ignore.untilDone();
        } catch (Throwable t) {
            if (t instanceof JxtaError) {
                throw (JxtaError) t;
            }
            
            if (LOG.isEnabledFor(Priority.ERROR))
                LOG.error( "Could not initialize graphical config dialog", t );
            
            BufferedReader in
            = new BufferedReader(new InputStreamReader(System.in));
            
            try {
                while (in.ready()) {
                    in.readLine();
                }
            } catch (Exception e) {
            }
            System.out.println("The window-based configurator does not seem"
            + " to be usable.");
            System.out.print("Do you want to stop and edit the current "
            + "configuration ? [no]: ");
            String answer = "no";
            try {
                answer = in.readLine();
            } catch (Exception e) {
            }
            
            // this will cover all the cases of the answer yes, while
            // allowing non-gui/batch type scripts to load the platform
            // see platform issue #45
            if ("yes".equalsIgnoreCase(answer)) {
                save();
                System.out.println("Exiting; edit the file \"" + configName
                + "\", remove the file \"reconf\", and then"
                + ", launch JXTA again.");
                throw new JxtaError("Manual Configuration Requested");
            } else {
                // Save that config.
                save();
                
                // save is done before RouterRdvMatch so that no original
                // data is removed from the saved config.
                routerRdvMatch();
                System.out.println("Using the current configuration.");
            }
        }
        
        // Set the global debug level.
        adjustLog4JPriority();
        
        // If password exists and is NOT
        // the default password, then
        // pop a dialog to get the password
        // Otherwise, we will pass the default
        // password as the password.
        checkPassword();
        
        // Configure fully TLS
        //  Set the password as a system property
        configureTls();
        
        // Add root certificate to the config adv
        addRootCert();
        
        // Save that config.
        save();
        
        // save is done before RouterRdvMatch so that no original data is
        // removed from the saved config.
        routerRdvMatch();
    }
    
    private void checkPassword() {
        //    If the passwd exists, then validate it.
        //
        //    We are called for three cases:
        //    1) Initial configuration of everything
        //           passwd file does not exist
        //    2) REconfiguration:  passwd file exists
        //    3) REstarting the platfrom: passwd file exists
        
        String password = System.getProperty("net.jxta.tls.password");
        String principal = System.getProperty("net.jxta.tls.principal");
        if (password != null && principal != null)
            return;
        
        if (JTlsUtil.passwdExists()) {
            try {
                new PasswordDialog();
            } catch (Throwable t) {
                System.out.println("The window-based authenticator does not "
                + "seem to be usable.");
                System.out.println("You must authenticate through the "
                + "command-line.");
                throw new JxtaError("Could not authenticate.");
            }
        }
    }
    
    private void configureTls() {
        
        // We configure TLS after the configurator is done.
        //   Still waiting for the password GUI
        String peerName = advertisement.getName();
        String password = System.getProperty("net.jxta.tls.password");
        String principal = System.getProperty("net.jxta.tls.principal");
        
        // we done with it, rid of it
        System.setProperty("net.jxta.tls.password", "");
        
        // Both the principal and the password are always
        // set either by the configuration panel or
        // security login.
        TlsConfig.init(principal, password);
        
    }
    
    // add root certificate to the config Advertisement
    public void addRootCert() {
        
        StructuredTextDocument paramDoc =
        (StructuredTextDocument)StructuredDocumentFactory.newStructuredDocument(
        new MimeMediaType("text", "xml"),
        "Parm");
        String rootCert = null;
        
        try {
            rootCert = TlsConfig.getRootCert();
        } catch (IOException e) {
            
            // This should never happen unless the file system is trashed.
            if (LOG.isEnabledFor(Priority.ERROR)) LOG.error("Cannot get root cert, Exception = ", e);
            
            return;
        }
        
        Element e = paramDoc.createElement("RootCert", rootCert);
        paramDoc.appendChild(e);
        advertisement.putServiceParam(PeerGroup.peerGroupClassID, paramDoc);
        
    }
    
    class PasswordDialog extends javax.swing.JFrame implements ActionListener {
        
        TextField principalField = new TextField("", 20);
        TextField passwordField = new TextField(20);
        
        private boolean principalDone = false;
        private boolean passwordDone = false;
        
        /**
         *  Dialog to prompt for a password
         */
        public PasswordDialog() {
            
            super("JXTA Secure Login");
            
            javax.swing.JPanel contentPane = new javax.swing.JPanel(new java.awt.BorderLayout());
            
            // principal
            javax.swing.JPanel cp0 = new javax.swing.JPanel(new java.awt.BorderLayout());
            javax.swing.JLabel pLabel = new javax.swing.JLabel("Secure Username: ");
            
            cp0.setBorder(javax.swing.BorderFactory.createEmptyBorder(20, 20, 20, 20));
            cp0.add(pLabel, java.awt.BorderLayout.WEST);
            cp0.add(principalField, java.awt.BorderLayout.EAST);
            
            // password
            javax.swing.JPanel cp1 = new javax.swing.JPanel(new java.awt.BorderLayout());
            javax.swing.JLabel label = new javax.swing.JLabel("Password: ");
            passwordField.setEchoChar('*');
            
            cp1.setBorder(javax.swing.BorderFactory.createEmptyBorder(20, 20, 20, 20));
            cp1.add(label, java.awt.BorderLayout.WEST);
            cp1.add(passwordField, java.awt.BorderLayout.EAST);
            
            principalField.addActionListener(this);
            passwordField.addActionListener(this);
            
            // panes in the pane
            contentPane.add(cp0,java.awt.BorderLayout.NORTH);
            contentPane.add(cp1,java.awt.BorderLayout.SOUTH);
            
            setContentPane(contentPane);
            
            java.awt.Rectangle screenRect = getGraphicsConfiguration().getBounds();
            setLocation(
            screenRect.x + screenRect.width / 2 - getSize().width / 2,
            screenRect.y + screenRect.height / 2 - getSize().height / 2);
            pack();
            setVisible(true);
            /* // commenting the following due to the fact that
            // dialog disappears debug, leaving the code in to debug later
            addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            try {
            int reply = javax.swing.JOptionPane.showConfirmDialog(frame,
            "Do you really want to exit?",
            "JXTA Secure Login",
            javax.swing.JOptionPane.YES_NO_OPTION,
            javax.swing.JOptionPane.QUESTION_MESSAGE);
            if (reply == javax.swing.JOptionPane.YES_OPTION) {
            setVisible(false);
            dispose();
            System.exit(0);
            }
            } catch (Exception any) {
            }
            }
            });
             */
            untilDone();
            dispose();
        }
        
        private synchronized void untilDone() {
            // wait until correct info entered
            try {
                wait();
            } catch (InterruptedException e) {
            }
        }
        
        public synchronized void actionPerformed(ActionEvent e) {
            
            String password = passwordField.getText();
            String principal = principalField.getText();
            
            if (principal.length() == 0 || !JTlsUtil.principalIsIssuer(principal) ){
                javax.swing.JOptionPane.showMessageDialog(this,
                "Invalid Secure Username. Try again.",
                "Error Message",
                javax.swing.JOptionPane.ERROR_MESSAGE);
                return;
            } else {
                System.setProperty("net.jxta.tls.principal", principal);
                if (password.length() == 0) {
                    passwordField.requestFocus();
                    return;
                }
            }
            
            if (password.length() == 0  || ! JTlsUtil.validPasswd(password) ){
                javax.swing.JOptionPane.showMessageDialog(this,
                "Invalid Password. Try again.",
                "Error Message",
                javax.swing.JOptionPane.ERROR_MESSAGE);
                return;
            } else {
                System.setProperty("net.jxta.tls.password",password);
                notifyAll();
            }
        }
    }
    
    private void routerRdvMatch() {
        
        try {
            
            // Make sure that the rendez-vous and router lists
            // are randomly ordered and reduce http rendez-vous and routers to their
            // intersection, in order to accomodate a current limitation.
            // This accounts for most of the junk below.
            // Note that if http routers are not required then all rdvs are good.
            Enumeration rdvElements = null;
            
            Vector rdvs = new Vector();
            try {
                TextElement param = (TextElement)
                advertisement.getServiceParam(PeerGroup.rendezvousClassID);
                if (param != null) {
                    // First, get the Rdv part of the service param.
                    rdvElements = param.getChildren("Rdv");
                    
                    // Get the rendezvous EndpointService Addresses
                    Enumeration rdvEnum = param.getChildren("Addr");
                    while (rdvEnum.hasMoreElements()) {
                        rdvs.addElement( ( (TextElement)
                        rdvEnum.nextElement() ).getTextValue() );
                    }
                }
            } catch (Exception nobigdeal) {
            }
            
            
            
            HTTPAdv httpAdv = null;
            boolean httpDisabled = false;
            Vector routers;
            try {
                Element param =
                advertisement.getServiceParam(PeerGroup.httpProtoClassID);
                if (param.getChildren("isOff").hasMoreElements()) {
                    httpDisabled = true;
                }
                
                // FIXME 20011220 bondolo@jxta.org Temporarily accept both nodes of
                // type HTTPAdv and TransportAdvertisement.
                Enumeration httpChilds = param.getChildren(
                TransportAdvertisement.getAdvertisementType());
                
                // get the TransportAdv from either TransportAdv or HttpAdv
                if( httpChilds.hasMoreElements() ) {
                    param = (Element) httpChilds.nextElement();
                } else {
                    httpChilds = param.getChildren(
                    HTTPAdv.getAdvertisementType());
                    
                    if( httpChilds.hasMoreElements() ) {
                        param = (Element) httpChilds.nextElement();
                    }
                }
                
                httpAdv = (HTTPAdv)
                AdvertisementFactory.newAdvertisement((TextElement) param);
            } catch (Exception nevermind) {
                // Very weird, though. We could not get the HttpAdv while we know
                // that at the very least it was created by fixAdv...
                httpAdv = new HTTPAdv( );
                httpAdv.setProtocol( "http" );
                httpAdv.setPort( "9700" );
                httpAdv.setRouterEnabled(true);
                httpAdv.setProxyEnabled(false);
                httpAdv.setServerEnabled(false);
                if (LOG.isEnabledFor(Priority.WARN)) LOG.warn("httpAdv has vanished");
            }
            
            routers = httpAdv.getRouters();
            if (routers == null) routers = new Vector();
            
            Vector intRouters = new Vector();
            Vector intRdvs = new Vector();
            int intersectSz = 0;
            PosRandom random = new PosRandom();
            
            Enumeration eRdvs = rdvs.elements();
            
            while (eRdvs.hasMoreElements()) {
                
                String rdv = (String) eRdvs.nextElement();
                
                // This elimination process is only for HTTP rdvs.
                // Tcp rdvs are letf alone; they're added at the end
                if (rdv.startsWith("tcp://")) {
                    continue;
                }
                rdv = rdv.substring(7);
                Enumeration eRouters = routers.elements();
                
                // If routing is not enabled, we do not have to look.
                // all rdvs are good (although probably not needed, but that's
                // not our decision to make).
                boolean found = true;
                if (httpAdv.getRouterEnabled()) {
                    found = false;
                    while (eRouters.hasMoreElements()) {
                        String router = (String) eRouters.nextElement();
                        if (rdv.equals(router)) {
                            found = true;
                            break;
                        }
                    }
                }
                if (!found) {
                    continue;
                }
                
                ++intersectSz;
                int here = random.nextPos() % intersectSz;
                
                intRouters.insertElementAt(rdv, here);
                intRdvs.insertElementAt("http://" + rdv, here);
            }
            
            // Add tcp rdvs at the begining or the end, dependent on weither
            // http is actually in use or not.
            int front;
            int range = 0;
            if (! httpDisabled) {
                front = intersectSz;
            } else {
                front = 0;
            }
            eRdvs = rdvs.elements();
            while (eRdvs.hasMoreElements()) {
                
                String rdv = (String) eRdvs.nextElement();
                if (rdv.startsWith("http://")) {
                    continue;
                } else {
                    ++range;
                    intRdvs.insertElementAt(rdv, random.nextPos() % range + front);
                }
            }
            
            // Replace the two vectors in the adv.
            if (intRouters.size() == 0) {
                intRouters = null;
            }
            
            httpAdv.setRouters(intRouters);
            StructuredTextDocument newRdvs = (StructuredTextDocument)
            StructuredDocumentFactory.newStructuredDocument(
            new MimeMediaType("text", "xml"),
            "Parm");
            
            // Copy back into the new document the Rendezvous flag
            if ((rdvElements != null) || rdvElements.hasMoreElements()) {
                StructuredDocumentUtils.copyElements(newRdvs,
                newRdvs,
                (Element) rdvElements.nextElement());
            }
            
            Enumeration rdvEnum = intRdvs.elements();
            while (rdvEnum.hasMoreElements()) {
                Element e =
                newRdvs.createElement("Addr", (String) rdvEnum.nextElement());
                newRdvs.appendChild(e);
            }
            
            StructuredDocument http =
            StructuredDocumentFactory.newStructuredDocument(
            new MimeMediaType("text", "xml"), "Parm");
            
            StructuredDocumentUtils.copyElements(
            http, http, (StructuredDocument)
            httpAdv.getDocument(new MimeMediaType("text", "xml")));
            
            if (httpDisabled) {
                http.appendChild(http.createElement("isOff"));
            }
            
            advertisement.putServiceParam(PeerGroup.httpProtoClassID, http);
            advertisement.putServiceParam(PeerGroup.rendezvousClassID,
            newRdvs);
        } catch(Exception keepGoing) {
        }
    }
    
    
    /**
     * Returns the advertisement.
     *
     * @return PAdvertisment the advertisement.
     * @since 1.0
     */
    public PeerAdvertisement get() {
        return advertisement;
    }
    
    
    /**
     * Save the advertisement in the standard config file.
     *
     * @since 1.0
     */
    public void save() {
        saveTo(configName);
    }
    
    /**
     * Load the advertisement from the standard config file.
     *
     * @since 1.0
     */
    public void load() {
        loadFrom(configName);
    }
    
    
    /**
     * Forces a manual reconf the next time the configurator is invoked.
     *
     * @since 1.0
     */
    public void setReconf() {
        try {
            File file = new File("reconf");
            new FileOutputStream(file).close();
        } catch (Exception ex1) {
            if (LOG.isEnabledFor(Priority.ERROR)) {
                LOG.error("Could not create reconfig flag.");
                LOG.error("Create the file reconfig by hand before retrying.");
            }
        }
    }
    
    /**
     * No manual reconf the next time the configurator is invoked.
     *
     * @since 1.0
     */
    public void clearReconf() {
        try {
            File file = new File("reconf");
            file.delete();
        } catch (Exception ex1) {
            if (LOG.isEnabledFor(Priority.ERROR)) {
                LOG.error("Could not remove reconfig flag.");
                LOG.error("Delete the file reconfig by hand before retrying.");
            }
        }
    }
    
    /**
     * Check if a manual reconf has been forced.
     *
     * @return boolean true if a manual reconf is being forced.
     * @since 1.0
     */
    public boolean isReconf() {
        try {
            boolean forceReconfig;
            
            File file = new File("reconf");
            forceReconfig = file.exists();
            
            if( forceReconfig && LOG.isEnabledFor(Priority.INFO) )
                LOG.info( "Reconfig requested - 'reconf' file found" );
            
            return forceReconfig;
        } catch (Exception ex1) {
            if (LOG.isEnabledFor(Priority.ERROR)){
                LOG.error("Could not check reconfig flag.");
                LOG.error("Assuming it exists.");
            }
            return true;
        }
    }
    
    
    /**
     * Loads the Peer Advertisement from the named file.
     *
     * @param  fileName  Name of the file where the adv is stored.
     * @since
     */
    private void loadFrom(String fileName) {
        
        FileInputStream advStream = null;
        File file = null;
        
        try {
            file = new File(fileName);
            advStream = new FileInputStream(file);
            
            advertisement = (PeerAdvertisement)
            AdvertisementFactory.newAdvertisement(
            new MimeMediaType("text/xml"), advStream);
            
            if (LOG.isEnabledFor(Priority.DEBUG)){
                LOG.debug("Recovered " + fileName);
            }
        } catch (FileNotFoundException e) {
            if (LOG.isEnabledFor(Priority.DEBUG)){
                LOG.debug("No Existing " + fileName);
            }
        } catch (Exception e) {
            if (LOG.isEnabledFor(Priority.WARN)) LOG.warn("Failed to Recover " + fileName);
            e.printStackTrace();
            try {
                // delete that bad file.
                file.delete();
            } catch (Exception ex1) {
                if (LOG.isEnabledFor(Priority.FATAL)) LOG.fatal("Could not remove " + fileName + ". Removed it by hand before retrying");
                throw new JxtaError("Could not remove " + fileName + ". Removed it by hand before retrying");
            }
            
            advertisement = null;
        }
        try {
            if (advStream != null) {
                advStream.close();
            }
        } catch (Exception e) {
        }
    }
    
    
    /**
     * Saves the peer advertisement.
     *
     * @param  fileName  The name of the file where to save it.
     * @since 1.0
     */
    private void saveTo(String fileName) {
        
        // Save the adv as input for future reconfiguration
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(fileName);
            Document aDoc =
            advertisement.getDocument(new MimeMediaType("text/xml"));
            aDoc.sendToStream(out);
        } catch (Exception e) {
            e.printStackTrace();
            if (LOG.isEnabledFor(Priority.WARN)) LOG.warn("Could not save to " + fileName);
        }
        try {
            out.close();
        } catch (Exception e) {
        }
    }
    
    // in the event win launcher sets a proxy parse it, and use it
    // returns http://proxyserver:####
    private String parseHttpProxyServer( String proxy ) {
        
        
        if ( proxy.indexOf(";") >= 0 ) {
            
            StringTokenizer tokenizer = new StringTokenizer(proxy,";");
            while (tokenizer.hasMoreElements() ) {
                String token = (String) tokenizer.nextElement();
                if (token.startsWith("http=") )
                    return token;
            }
        }
        return proxy;
    }
    /**
     * Makes sure the PeerAdvertisement is sane. If it is not, fix it.
     * If it does not exist at all, create a default one.
     *
     * @param  adv  Description of Parameter
     * @return      boolean true: manual reconf advised.
     * @since 1.0
     */
    private boolean fixAdvertisement() {
        
        try {
            
            boolean reconf = isReconf();
            
            if (advertisement == null) {
                reconf = true;
                
                if( LOG.isEnabledFor(Priority.INFO) )
                    LOG.info( "Reconfig requested - no PeerAdvertisement" );
                
                advertisement = (PeerAdvertisement)
                AdvertisementFactory.newAdvertisement(
                PeerAdvertisement.getAdvertisementType());
                
                advertisement.setName("");
                advertisement.setDescription("");
                advertisement.setPeerID(
                IDFactory.newPeerID(PeerGroupID.worldPeerGroupID));
                // Do not set a GID here; we have no authority to do so and it is
                // fortunately disregarded anyway.
                //            advertisement.setPeerGroupID(PeerGroupID.worldPeerGroupID);
            }
            
            /*
             *  create the default Service Params for protos.
             */
            StructuredDocument http = (StructuredTextDocument)
            advertisement.getServiceParam(PeerGroup.httpProtoClassID);
            StructuredDocument tcp  = (StructuredTextDocument)
            advertisement.getServiceParam(PeerGroup.tcpProtoClassID);
            
            HTTPAdv httpAdv =  null;
            boolean httpDisabled = false;
            TCPAdv  tcpAdv = null;
            boolean tcpDisabled = false;
            
            if (http != null) {
                try {
                    httpDisabled = http.getChildren("isOff").hasMoreElements();
                    
                    Element param = null;
                    
                    Enumeration httpChilds = http.getChildren(
                    TransportAdvertisement.getAdvertisementType());
                    
                    // get the TransportAdv from either TransportAdv or HttpAdv
                    if( httpChilds.hasMoreElements() ) {
                        param = (Element) httpChilds.nextElement();
                    } else {
                        httpChilds = http.getChildren(
                        HTTPAdv.getAdvertisementType());
                        
                        if( httpChilds.hasMoreElements() ) {
                            param = (Element) httpChilds.nextElement();
                        }
                    }
                    
                    httpAdv = (HTTPAdv)
                    AdvertisementFactory.newAdvertisement((TextElement) param);
                    boolean found = false;
                    try {
                        String interfaceAddress = httpAdv.getInterfaceAddress();
                        if (   interfaceAddress != null
                        && !interfaceAddress.equals("")) {
                            String hostAddr =
                            InetAddress.getLocalHost().getHostAddress();
                            InetAddress[] all = InetAddress.getAllByName(hostAddr);
                            int i = all.length;
                            while (i-- > 0) {
                                if (all[i].getHostAddress().equals(interfaceAddress)) {
                                    found = true;
                                    break;
                                    // OK, it is in-there
                                }
                            }
                        }
                    } catch (Throwable t2) {
                        // Could not verify ? Something's fishy. Let's check further
                        // if a reconfig is in order.
                    }
                    if (! found) {
                        String cfMode = httpAdv.getConfigMode();
                        if (cfMode == null || ! cfMode.equals("manual")) {
                            reconf = true;
                            
                            if( LOG.isEnabledFor(Priority.INFO) )
                                LOG.info( "Reconfig requested - http set to manual config" );
                        }
                    }
                } catch (Exception advTrouble) {
                    if( LOG.isEnabledFor(Priority.WARN) )
                        LOG.warn( "Reconfig requested - http advertisement corrupted", advTrouble );
                    
                    httpAdv = null;
                }
            }
            
            if (httpAdv == null) {
                reconf = true;
                if( LOG.isEnabledFor(Priority.INFO) )
                    LOG.info( "Reconfig requested - http advertisement missing" );
                
                httpAdv = new HTTPAdv( );
                httpAdv.setProtocol( "http" );
                httpAdv.setPort( "9700" );
                httpAdv.setRouterEnabled(true);
                httpAdv.setProxyEnabled(false);
                httpAdv.setServerEnabled(false);
            }
            
            if (tcp != null) {
                try {
                    tcpDisabled = tcp.getChildren("isOff").hasMoreElements();
                    
                    Element param = null;
                    
                    Enumeration tcpChilds = tcp.getChildren(
                    TransportAdvertisement.getAdvertisementType());
                    
                    // get the TransportAdv from either TransportAdv or TcpAdv
                    if( tcpChilds.hasMoreElements() ) {
                        param = (Element) tcpChilds.nextElement();
                    } else {
                        tcpChilds = tcp.getChildren(
                        TCPAdv.getAdvertisementType());
                        
                        if( tcpChilds.hasMoreElements() ) {
                            param = (Element) tcpChilds.nextElement();
                        }
                    }
                    
                    tcpAdv = (TCPAdv)
                    AdvertisementFactory.newAdvertisement((TextElement) param);
                    boolean found = false;
                    try {
                        String interfaceAddress = tcpAdv.getInterfaceAddress();
                        if (    interfaceAddress != null
                        && !interfaceAddress.equals("")) {
                            String hostAddr =
                            InetAddress.getLocalHost().getHostAddress();
                            InetAddress[] all = InetAddress.getAllByName(hostAddr);
                            int i = all.length;
                            while (i-- > 0) {
                                if (all[i].getHostAddress().equals(interfaceAddress)) {
                                    found = true;
                                    break;
                                    // OK, it is in-there
                                }
                            }
                        }
                    } catch (Throwable t2) {
                        // Could not verify ? Something's fishy. Let's check further
                        // if a reconfig is in order.
                    }
                    if (! found) {
                        String cfMode = tcpAdv.getConfigMode();
                        if (cfMode == null || ! cfMode.equals("manual")) {
                            reconf = true;
                            
                            if( LOG.isEnabledFor(Priority.INFO) )
                                LOG.info( "Reconfig requested - tcp set to manual config" );
                        }
                    }
                } catch (Exception advTrouble) {
                    if( LOG.isEnabledFor(Priority.WARN) )
                        LOG.warn( "Reconfig requested - http advertisement corrupted", advTrouble );
                    
                    tcpAdv = null;
                }
            }
            
            if (tcpAdv == null) {
                reconf = true;
                if( LOG.isEnabledFor(Priority.INFO) )
                    LOG.info( "Reconfig requested - tcp advertisement missing" );
                
                String port = "9701";
                // get the port from a property
                String tcpPort = System.getProperty("jxta.tcp.port");
                if (tcpPort != null)
                    port = tcpPort;
                
                tcpAdv = new TCPAdv();
                
                tcpAdv.setProtocol( "TCP" );
                tcpAdv.setPort( port );
                tcpAdv.setMulticastAddr( "224.0.1.85" );
                tcpAdv.setMulticastPort( "1234" );
                tcpAdv.setMulticastSize( "16384" );
                tcpAdv.setServer( "" );
                tcpAdv.setInterfaceAddress( "" );
            }
            
            String hn = "";
            try {
                hn = InetAddress.getLocalHost().getHostName();
            } catch (Exception e) {
                hn = "";
            }
            
            if (httpAdv.getServer() == null) {
                httpAdv.setServer(hn + ":9700");
                httpAdv.setServerEnabled(false);
            }
            
            String winProxy = System.getProperty("jxta.proxy");
            if (winProxy != null) {
                String proxyServer = parseHttpProxyServer(winProxy);
                
                if (proxyServer != null) {
                    httpAdv.setProxyEnabled(true);
                    httpAdv.setProxy(proxyServer.substring(proxyServer.indexOf("=")+1));
                }
            } else if (httpAdv.getProxy() == null) {
                httpAdv.setProxy("myProxy.myDomain:8080");
                httpAdv.setProxyEnabled(false);
            }
            
            if (httpAdv.getRouters() == null) {
                Vector r = new Vector();
                httpAdv.setRouters(r);
                // Do not set routerEnabled. If the adv is brand new, then
                // it was set to true already (as a default). Otherwise it is
                // what it is. HTTPAdv already set it properly;  we're just
                // making ConfigDialog's life simpler by providing an empty
                // vector instead of null.
            }
            
            // See if TLS will reconfigure
            if (TlsConfig.willConfigureTls()) {
                reconf = true;
                
                if( LOG.isEnabledFor(Priority.INFO) )
                    LOG.info( "Reconfig requested - TLS wanted config" );
                
            }
            
            // If we did not modify anything of importance or see anything wrong,
            // leave the adv alone.
            if (reconf == false) return false;
            
            
            // Create new param docs that contain the updated advs
            http = StructuredDocumentFactory.newStructuredDocument(
            new MimeMediaType("text", "xml"), "Parm");
            StructuredDocumentUtils.copyElements(
            http,
            http,
            (StructuredDocument) httpAdv.getDocument(new MimeMediaType("text", "xml")));
            if (httpDisabled) {
                http.appendChild(http.createElement("isOff"));
            }
            advertisement.putServiceParam(PeerGroup.httpProtoClassID, http);
            
            tcp = StructuredDocumentFactory.newStructuredDocument(
            new MimeMediaType("text", "xml"), "Parm");
            
            StructuredDocumentUtils.copyElements(
            tcp,
            tcp,
            (StructuredDocument) tcpAdv.getDocument(new MimeMediaType("text", "xml")));
            if (tcpDisabled) {
                tcp.appendChild(tcp.createElement("isOff"));
            }
            advertisement.putServiceParam(PeerGroup.tcpProtoClassID, tcp);
        } catch(Exception serious) {
            if (LOG.isEnabledFor(Priority.ERROR))
                LOG.error("Trouble while fixing advertisement. Hope for the best.", serious);
        }
        
        return true;
        
    }
    
    /**
     * Adjust the log4j priority based on the user's configuration file.
     * If the configuration is not set or the value is "user default",
     * then don't change it.
     *
     * @since 1.0
     */
    private void adjustLog4JPriority() {
        if (advertisement == null || advertisement.getDebugLevel() == null) {
            return;
        }
        
        String requestedLevel = advertisement.getDebugLevel();
        if ("user default".equals(requestedLevel)) {
            if (LOG.isEnabledFor(Priority.DEBUG)) LOG.debug("user default requested");
            return;
        }
        
        if (LOG.isEnabledFor(Priority.DEBUG)) LOG.debug("Setting priority to [" + requestedLevel +
        "] based on the user's configuration");
        Category jxtaCategory = Category.getInstance("net.jxta");
        jxtaCategory.setPriority(Priority.toPriority(requestedLevel));
    }
    
    
    /**
     *Description of the Class
     */
    class PosRandom extends Random {
        /**
         *Description of the Method
         *
         * @return    Description of the Returned Value
         * @since 1.0
         */
        public int nextPos() {
            return next(31);
        }
    }
}
